/*
 * SPDX-FileCopyrightText: 2016 Weng Xuetian <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-only
 *
 */

#ifndef XIMCOMMON_H
#define XIMCOMMON_H

#include "xcbimdkit_export.h"

#ifndef XCBIMDKIT_DECL_BEGIN
#ifdef __cplusplus
#define XCBIMDKIT_DECL_BEGIN extern "C" {
#else
#define XCBIMDKIT_DECL_BEGIN
#endif
#endif

#ifndef XCBIMDKIT_DECL_END
#ifdef __cplusplus
#define XCBIMDKIT_DECL_END }
#else
#define XCBIMDKIT_DECL_END
#endif
#endif

#endif
